/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * SetBoundaryConditionsAction.cpp
 *
 *  Created on: Jan 2, 2012
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include <sstream>
#include <vector>

#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "Box.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include "Actions/WorldAction/SetBoundaryConditionsAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "SetBoundaryConditionsAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr WorldSetBoundaryConditionsAction::performCall() {
  // gather undo information
  std::stringstream undostream;
  undostream << World::getInstance().getDomain().getConditionNames();

  // create undo state
  WorldSetBoundaryConditionsState *UndoState =
      new WorldSetBoundaryConditionsState(
          undostream.str(),
          params
          );

  // set conditions
  World::getInstance().getDomain().setConditions(params.newconditions.get());

  LOG(0, "STATUS: Boundary conditions are now " << World::getInstance().getDomain().getConditionNames() << ".");
  return ActionState::ptr(UndoState);
}

ActionState::ptr WorldSetBoundaryConditionsAction::performUndo(ActionState::ptr _state) {
  WorldSetBoundaryConditionsState *state = assert_cast<WorldSetBoundaryConditionsState*>(_state.get());

  // set old conditions
  World::getInstance().getDomain().setConditions(state->undostate);

  LOG(0, "STATUS: Boundary conditions are set back to " << World::getInstance().getDomain().getConditionNames() << ".");
  return ActionState::ptr(_state);
}

ActionState::ptr WorldSetBoundaryConditionsAction::performRedo(ActionState::ptr _state){
  WorldSetBoundaryConditionsState *state = assert_cast<WorldSetBoundaryConditionsState*>(_state.get());

  // set again conditions
  World::getInstance().getDomain().setConditions(state->params.newconditions.get());

  LOG(0, "STATUS: Boundary conditions are again " << World::getInstance().getDomain().getConditionNames() << ".");
  return ActionState::ptr(_state);
}

bool WorldSetBoundaryConditionsAction::canUndo() {
  return true;
}

bool WorldSetBoundaryConditionsAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
