/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * SetDefaultNameAction.cpp
 *
 *  Created on: May 8, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

#include "Actions/WorldAction/SetDefaultNameAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "SetDefaultNameAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr WorldSetDefaultNameAction::performCall() {
  string Worldsdefaultname;
  Worldsdefaultname = World::getInstance().getDefaultName();
  WorldSetDefaultNameState *UndoState = new WorldSetDefaultNameState(Worldsdefaultname, params);

  World::getInstance().setDefaultName(params.newname.get());
  LOG(0, "Default name of new molecules set to " << World::getInstance().getDefaultName() << ".");
  return ActionState::ptr(UndoState);
}

ActionState::ptr WorldSetDefaultNameAction::performUndo(ActionState::ptr _state) {
  WorldSetDefaultNameState *state = assert_cast<WorldSetDefaultNameState*>(_state.get());

  World::getInstance().setDefaultName(state->lastname);
  LOG(0, "Default name of new molecules set to " << World::getInstance().getDefaultName() << ".");

  return ActionState::ptr(_state);
}

ActionState::ptr WorldSetDefaultNameAction::performRedo(ActionState::ptr _state){
  WorldSetDefaultNameState *state = assert_cast<WorldSetDefaultNameState*>(_state.get());

  World::getInstance().setDefaultName(state->params.newname.get());
  LOG(0, "Default name of new molecules set to " << World::getInstance().getDefaultName() << ".");

  return ActionState::ptr(_state);
}

bool WorldSetDefaultNameAction::canUndo() {
  return true;
}

bool WorldSetDefaultNameAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
